@@ -24,6 +24,7 @@ from beets import ui
 from beets import util
 from beets import config
 from beets import mediafile
+import mutagen
 
 _MUTAGEN_FORMATS = {
     'asf': 'ASF',
@@ -106,7 +107,7 @@ class ScrubPlugin(BeetsPlugin):
                 for tag in f.keys():
                     del f[tag]
                 f.save()
-            except IOError as exc:
+            except (IOError, mutagen.MutagenError) as exc:
                 self._log.error(u'could not scrub {0}: {1}',
                                 util.displayable_path(path), exc)
 
@@ -119,10 +120,11 @@ class ScrubPlugin(BeetsPlugin):
             try:
                 mf = mediafile.MediaFile(util.syspath(item.path),
                                          config['id3v23'].get(bool))
-            except IOError as exc:
+            except mediafile.UnreadableFileError as exc:
                 self._log.error(u'could not open file to scrub: {0}',
                                 exc)
-            art = mf.art
+                return
+            images = mf.images
 
         # Remove all tags.
         self._scrub(item.path)
@@ -131,12 +133,15 @@ class ScrubPlugin(BeetsPlugin):
         if restore:
             self._log.debug(u'writing new tags after scrub')
             item.try_write()
-            if art:
+            if images:
                 self._log.debug(u'restoring art')
-                mf = mediafile.MediaFile(util.syspath(item.path),
-                                         config['id3v23'].get(bool))
-                mf.art = art
-                mf.save()
+                try:
+                    mf = mediafile.MediaFile(util.syspath(item.path),
+                                             config['id3v23'].get(bool))
+                    mf.images = images
+                    mf.save()
+                except mediafile.UnreadableFileError as exc:
+                    self._log.error(u'could not write tags: {0}', exc)
 
     def import_task_files(self, session, task):
         """Automatically scrub imported files."""
