@@ -66,7 +66,6 @@ def library_data(lib, args):
 def library_data_emitter(item):
     def emitter():
         data = dict(item.formatted())
-        data.pop('path', None)  # path is fetched from item
 
         return data, item
     return emitter
@@ -184,6 +183,8 @@ class InfoPlugin(BeetsPlugin):
         included_keys = []
         for keys in opts.included_keys:
             included_keys.extend(keys.split(','))
+        # Drop path even if user provides it multiple times
+        included_keys = [k for k in included_keys if k != 'path']
         key_filter = make_key_filter(included_keys)
 
         first = True
@@ -219,8 +220,13 @@ def make_key_filter(include):
     dictionary that only includes the key-value pairs where the key
     glob-matches one of the keys in `include`.
     """
+    # By default, if no field inclusions are specified, include
+    # everything but `path`.
     if not include:
-        return identity
+        def filter_(data):
+            return {k: v for k, v in data.items()
+                    if k != 'path'}
+        return filter_
 
     matchers = []
     for key in include:
@@ -236,7 +242,3 @@ def make_key_filter(include):
         return filtered
 
     return filter_
-
-
-def identity(val):
-    return val
